/**************************************************************************/
/*																		  */ 
/*			  Copyright (c) 1999-2000 by Atmel Corporation				  */ 
/*																		  */ 
/*	This software is copyrighted by and is the sole property of Atmel	  */ 
/*	Corporation.  All rights, title, ownership, or other interests		  */ 
/*	in the software remain the property of Atmel Corporation.  This 	  */ 
/*	software may only be used in accordance with the corresponding		  */ 
/*	license agreement.	Any un-authorized use, duplication, transmission, */ 
/*	distribution, or disclosure of this software is expressly forbidden.  */ 
/*																		  */
/*	This Copyright notice may not be removed or modified without prior	  */ 
/*	written consent of Atmel Corporation.								  */ 
/*																		  */ 
/*	Atmel Corporation, Inc. reserves the right to modify this software	  */ 
/*	without notice. 													  */ 
/*																		  */ 
/*	Atmel Corporation.													  */
/*	2325 Orchard Parkway			   literature@atmel.com 			  */
/*	San Jose, CA 95131				   http://www.atmel.com 			  */
/*																		  */
/**************************************************************************/


/**************************************************************************/
/**************************************************************************/
/** 																	  */ 
/** FastVNET (PCMCIA) NDIS Miniport Driver								  */ 
/** 																	  */
/**   Standard Include File 											  */
/** 																	  */
/**************************************************************************/
/**************************************************************************/



#ifndef __stdinclude_h_OK__
#define __stdinclude_h_OK__


#ifndef VNet_502_API
#define VNet_502_API
#endif

#define OUT_OF_RANGE_TIME_OUT				2000
#define ASSOCIATION_RESPONSE_TIME_OUT		1000
#define JOIN_TIME_OUT						1000

#define MAX_AUTHENTICATION_RETRIES			3
#define MAX_ASSOCIATION_RETRIES 			3

//
// Offsets of some fields in frames
//
#define MGMT_FRAME_SA_OFFSET						10
#define DATA_FRAME_ADHOC_SA_OFFSET					10
#define DATA_FRAME_INFR_SA_OFFSET					16
#define DATA_FRAME_ADHOC_ADR3_OFFSET				16

#define BEACON_INTERVAL_OFFSET						8			//12+2
#define SSID_ELLEMENT_OFFSET						14			//12+2
#define SSID_ELLEMENT_LENGTH_OFFSET 				13			//12+1
#define SUP_RATES_ELLEMENT_LENGTH_REL_OFFSET		15			//12+2+1
#define DS_PARAM_SET_ELLEMENT_REL_OFFSET			18			//12+2+2+2
#define BSSID_HEADER_OFFSET 						16
#define MGMT_FRAME_BODY_OFFSET						24

#define WIRELESS_HEADER_ADDRESS3_OFFSET 			16
#define CAPABILITY_INFO_IN_BEACON_OFFSET			10			
#define STATUS_IN_AUTHENTICATION_OFFSET 			4			
#define TRANS_SEQ_NO_IN_AUTHENTICATION_OFFSET		2			
#define CHALL_TEXT_EL_ID_IN_AUTHENTICATION_OFFSET	6			
#define CHALL_TEXT_LENGTH_IN_AUTHENTICATION_OFFSET	7			
#define CHALL_TEXT_IN_AUTHENTICATION_OFFSET 		8

#define STATUS_IN_ASSOCIATION_RESP_OFFSET			2			
#define ASS_ID_IN_ASSOCIATION_RESP_OFFSET			4
#define SUP_RATES_LENGTH_IN_ASSOCIATION_RESP_OFFSET 7
#define SUP_RATES_IN_ASSOCIATION_RESP_OFFSET		8

//////////////////////////////////
//	Functions in card.c
//////////////////////////////////

BOOLEAN
CardReset(PVNet_ADAPTER Adapter);

BOOLEAN 
CardGetHostInfo(PVNet_ADAPTER Adapter);

BOOLEAN 
CardStartUp(PVNet_ADAPTER Adapter);

VOID
CardStop(PVNet_ADAPTER Adapter);


//////////////////////////////////
//	Functions in command.c
//////////////////////////////////

UCHAR 
SendCommand(	PVNet_ADAPTER Adapter, 
				UCHAR CmdId, 
				PUCHAR pParameters);			

BOOLEAN
SetChannel(PVNet_ADAPTER Adapter, UCHAR Channel);

BOOLEAN
SetDataFramesTxRateMode(PVNet_ADAPTER Adapter, UCHAR RateIndex);

BOOLEAN
SetRtsThreshold(PVNet_ADAPTER Adapter, USHORT Threshold);

BOOLEAN
SetFragmentationThreshold(PVNet_ADAPTER Adapter, USHORT Threshold);

BOOLEAN
SetPreambleType(PVNet_ADAPTER Adapter, UCHAR PreambleType);

BOOLEAN
SetCtrlFramesTxRate(PVNet_ADAPTER Adapter);

BOOLEAN
SetMgmtFramesTxRate(PVNet_ADAPTER Adapter);

BOOLEAN 
GetStatistics(PVNet_ADAPTER Adapter, PUCHAR Statistics);

BOOLEAN
SetPowerMgmtMode(PVNet_ADAPTER Adapter, UCHAR PSmode);

BOOLEAN
SetAssociationID(PVNet_ADAPTER Adapter, USHORT AssocID);

BOOLEAN
SetSupportedRates(PVNet_ADAPTER Adapter, PUCHAR SupportedRates, UCHAR SupRatesLength);

UCHAR
Join(PVNet_ADAPTER Adapter, UCHAR Type);

UCHAR
Start(PVNet_ADAPTER Adapter, UCHAR Type);

BOOLEAN
Scan(PVNet_ADAPTER Adapter, BOOLEAN UseSpecificSSID);	

VOID
CommandCompleteDPC(PVNet_ADAPTER Adapter);

BOOLEAN
SetPromiscuousMode(PVNet_ADAPTER Adapter, UCHAR Mode);

BOOLEAN
SetMACaddress(PVNet_ADAPTER Adapter, PUCHAR MACaddress);


BOOLEAN
SetMibValues(PVNet_ADAPTER Adapter);

BOOLEAN 
GetBSSID(PVNet_ADAPTER Adapter, PUCHAR bssid);

BOOLEAN
SetBeaconPeriod(PVNet_ADAPTER Adapter, USHORT BeaconPeriod);


#ifdef RFMD

BOOLEAN
GetRegulatoryDomain(PVNet_ADAPTER Adapter, PUCHAR RegDomain);

UCHAR
DisableRadio(PVNet_ADAPTER Adapter);

UCHAR
EnableRadio(PVNet_ADAPTER Adapter);

BOOLEAN 
SetRegulatoryDomain(PVNet_ADAPTER Adapter, UCHAR RegDomainCode);

#else

BOOLEAN
SetSSIDsize(PVNet_ADAPTER Adapter, UCHAR size);

#endif

//////////////////////////////////
//	Functions in flash.c
//////////////////////////////////

VOID 
WriteFlash8(PVNet_ADAPTER Adapter, ULONG Address, UCHAR Data);

VOID 
ReadFlash8(PVNet_ADAPTER Adapter, ULONG Address, PUCHAR pData);


BOOLEAN 
GetFlashSize(PVNet_ADAPTER Adapter);

VOID
GetFlashID(PVNet_ADAPTER Adapter, PUCHAR ManID, PUCHAR DevID);

VOID
EraseFlash(PVNet_ADAPTER Adapter);

BOOLEAN
BlankCheck(PVNet_ADAPTER Adapter);

BOOLEAN
DownloadBufferToFlash(PVNet_ADAPTER Adapter, ULONG bytes, PUCHAR data, ULONG sect_offset);

BOOLEAN
ProgramFlash(PVNet_ADAPTER Adapter);

VOID
GetCurrentCardInfo(PVNet_ADAPTER Adapter);

VOID 
EnterFlashIDMode(PVNet_ADAPTER Adapter); 


/////////////////////////////////////////////
// Functions in vnetwin.c, oidsman.c (Windows specific)
/////////////////////////////////////////////
#ifdef WINDOWS_OS
VNet_502_API NTSTATUS
DriverEntry(
	IN PDRIVER_OBJECT DriverObject,
	IN PUNICODE_STRING RegistryPath
	);


VNet_502_API NDIS_STATUS
VNetInitialize(
	OUT PNDIS_STATUS OpenErrorStatus,
	OUT PUINT SelectedMediumIndex,
	IN PNDIS_MEDIUM MediumArray,
	IN UINT MediumArraySize,
	IN NDIS_HANDLE MiniportAdapterHandle,
	IN NDIS_HANDLE ConfigurationHandle
	);

VNet_502_API VOID
VNetIsr(
			OUT PBOOLEAN InterruptRecognized,	
			OUT PBOOLEAN QueueDpc,				
			IN PVOID Context);
VNet_502_API VOID
VNetHandleInterrupt(
	IN NDIS_HANDLE MiniportAdapterContext);

VNet_502_API NDIS_STATUS
VNetTransferData(
	OUT PNDIS_PACKET Packet,
	OUT PUINT BytesTransferred,
	IN NDIS_HANDLE MiniportAdapterContext,
	IN NDIS_HANDLE MiniportReceiveContext,	
	IN UINT ByteOffset, 					
	IN UINT BytesToTransfer 				
	);

NDIS_STATUS
RegisterAdapter(
	IN PVNet_ADAPTER Adapter,			
	IN NDIS_HANDLE ConfigurationHandle	
	);


VNet_502_API NDIS_STATUS
VNetSend(
	IN NDIS_HANDLE MiniportAdapterContext,
	IN PNDIS_PACKET Packet,
	IN UINT Flags
	);

#ifndef WINNT4
NDIS_STATUS 
GetCardResources(
		IN NDIS_HANDLE WrapperConfigurationContext, 
		IN PVNet_ADAPTER Adapter);
#endif

#if defined (WINNT4) && defined (PCI)
NDIS_STATUS
GetNT4PCICardResources(
					   PVNet_ADAPTER Adapter);
#endif

NDIS_STATUS 
ParseRegistryParameters(	PVNet_ADAPTER Adapter, 
							NDIS_HANDLE ConfigurationHandle);

NDIS_STATUS
VNetReset(
	OUT PBOOLEAN AddressingReset,			
	IN NDIS_HANDLE MiniportAdapterContext	
	);

VOID
MgmtTimeOutHandler( PVOID SysSpec1,
				NDIS_HANDLE MiniportAdapterContext,
				PVOID SysSpec2, 
				PVOID SysSpec3);

VNet_502_API VOID
VNetHalt(
	IN NDIS_HANDLE MiniportAdapterContext);



//	Functions in oidsman.c


NDIS_STATUS
VNetQueryInformation(
	IN NDIS_HANDLE MiniportAdapterContext,	
	IN NDIS_OID Oid,						
	IN PVOID InformationBuffer, 			
	IN ULONG InformationBufferLength,		
	OUT PULONG BytesWritten,				
	OUT PULONG BytesNeeded					
											
);


NDIS_STATUS
VNetSetInformation(
	IN NDIS_HANDLE MiniportAdapterContext,	
	IN NDIS_OID Oid,						
	IN PVOID InformationBuffer, 			
	IN ULONG InformationBufferLength,		
	OUT PULONG BytesRead,					
	OUT PULONG BytesNeeded					
	);										


VOID 
ReleaseResources( IN PVNet_ADAPTER Adapter );


VOID
RejectPendingTxPackets(Adapter);

VOID
PeriodicTimerHandler(	PVOID SysSpec1,
				NDIS_HANDLE MiniportAdapterContext,
				PVOID SysSpec2, 
				PVOID SysSpec3);

#ifdef WINXP

VOID VNetPnPEventNotify(
	IN	NDIS_HANDLE 			MiniportAdapterContext,
	IN	NDIS_DEVICE_PNP_EVENT	PnPEvent,
	IN	PVOID					InformationBuffer,
	IN	ULONG					InformationBufferLength);

VOID VNetShutdown(
	IN	NDIS_HANDLE MiniportAdapterContext);

#endif //WINXP

#endif //WINDOWS_OS

//////////////////////////////////
//	Functions in mgmt.c
//////////////////////////////////

BOOLEAN 
SendAuthenticationRequest(PVNet_ADAPTER Adapter, PUCHAR pChallengeText, UCHAR ChallengeTextLength);

BOOLEAN
SendAssociationRequest(PVNet_ADAPTER Adapter);

BOOLEAN
SendReAssociationRequest(PVNet_ADAPTER Adapter);

BOOLEAN 
TransmitMgmtFrame(	PVNet_ADAPTER Adapter, 
					PUCHAR pFrameBuffer, 
					USHORT	FrameSize);

VOID 
MgmtFrameRxProcessing(	PVNet_ADAPTER Adapter, 
							UCHAR FrameSubtype,
							USHORT FrameLength);


VOID
DisplayMgmtFrameStatus(ULONG StatusCode);

BOOLEAN
StoreBSSinfo(PVNet_ADAPTER Adapter, PUCHAR BSSID, PUCHAR pBeaconOrPrope, UCHAR Channel, BOOLEAN IsProbe);

int
RetrieveBSS(PVNet_ADAPTER Adapter, PUCHAR BSSID);

VOID
SelectBSS(PVNet_ADAPTER Adapter, int APindex);

//////////////////////////////////
//	Functions in rx.h
//////////////////////////////////

VOID 
RxInit(PVNet_ADAPTER Adapter);

#ifdef SNIFFER
USHORT
FrameFiltering(PVNet_ADAPTER Adapter, UCHAR* rFrame);

USHORT 
FrameAddressFiltering(PVNet_ADAPTER Adapter, UCHAR* rFrame);
#endif

VOID 
RxCompleteDPC(PVNet_ADAPTER Adapter);

BOOLEAN 
GetChannelFromBeaconOrProbe(PUCHAR pBeaconOrProbeFrame, PUCHAR pChannel);

void 
GetRxDescRssi(PVNet_ADAPTER Adapter, USHORT Descriptor, PCHAR rssi);

//////////////////////////////////
//	Functions in tx.h
//////////////////////////////////

VOID 
TxInit(PVNet_ADAPTER Adapter);

VOID 
TxCompleteDPC(PVNet_ADAPTER Adapter);

VOID
TxUpdateDescriptor(PVNet_ADAPTER Adapter, ULONG TotalBytes, USHORT StartOfTxBuffer, BOOLEAN IsData);

UCHAR 
TxEthernetPacket(PVNet_ADAPTER Adapter, 
					 PUCHAR pTxBuf, 
					 ULONG CurrentBytes, 
					 PULONG TotalBytes, 
					 USHORT StartOfTxBuf,
					 BOOLEAN ContainsHeader);

VOID 
TxIndicateCompletion(PVNet_ADAPTER Adapter);

BOOLEAN 
TxResourcesAvailable(PVNet_ADAPTER Adapter, ULONG Bytes, PUSHORT StartOfTxBuf);

//////////////////////////////////
//	Functions in vnet.c
//////////////////////////////////

BOOLEAN 
EnableDisableWEP(PVNet_ADAPTER Adapter, BOOLEAN WepIsOn);

BOOLEAN 
ConfigureWEP(PVNet_ADAPTER Adapter);

BOOLEAN
ResetAdapter(PVNet_ADAPTER Adapter);

VOID
InitAdapter(PVNet_ADAPTER Adapter);

BOOLEAN
InitAndStartCard(PVNet_ADAPTER Adapter);

VOID 
CopyCodeToInternalMemory(PVNet_ADAPTER Adapter);

BOOLEAN
ChangeConfiguration(PVNet_ADAPTER Adapter, PDEVICE_CONFIGURATION pHostInfo);


VOID
GetMACaddress(PVNet_ADAPTER Adapter);

#ifdef WINDOWS_OS
BOOLEAN 
GetFwFromFile(PVNet_ADAPTER Adapter,NDIS_STRING FWFileName);
#endif

BOOLEAN 
GetDefaultFw(PVNet_ADAPTER Adapter);

VOID 
ProcessInterrupt(PVNet_ADAPTER Adapter);

VOID 
MgmtTimeOutCallBack(PVNet_ADAPTER Adapter);

UCHAR 
AsciiToByte(UCHAR Ascii1, UCHAR Ascii2);

VOID
ByteToAscii(UCHAR Byte, PUCHAR Ascii1, PUCHAR Ascii2);

BOOLEAN 
JoinSpecificBSS(PVNet_ADAPTER Adapter, int BSSindex, BOOLEAN ChangeInitialSSID);

VOID 
VNetZeroMemory(PUCHAR buf, ULONG bytes);

BOOLEAN
ValidateChannelForDomain(PVNet_ADAPTER Adapter, PUCHAR pChannel);

#ifdef INT_ROAM
BOOLEAN
SetIntRoaming(PVNet_ADAPTER Adapter);

BOOLEAN 
GetMultiDomainMIB(PVNet_ADAPTER Adapter);

#endif

#ifdef LINUX_OS
void MgmtTimer(ulong a);
void SetMgmtTimer(PVNet_ADAPTER Adapter);
UINT
VNetSend(PVNet_ADAPTER Adapter, PUCHAR Packet, ULONG bytes);
VOID
SetParameters(PVNet_ADAPTER Adapter);
#endif

////////
// tx.c
////////
VOID
TxInit(PVNet_ADAPTER Adapter);

VOID
TxCompleteDPC(PVNet_ADAPTER Adapter);

UCHAR
TxEthernetPacket(PVNet_ADAPTER Adapter,
					 PUCHAR pTxBuf,
					 ULONG CurrentBytes,
					 PULONG TotalBytes,
					 USHORT StartOfTxBuf,
					 BOOLEAN ContainsHeader);

BOOLEAN 
TxResourcesAvailable(PVNet_ADAPTER Adapter, ULONG Bytes, PUSHORT StartOfTxBuf);

VOID
TxUpdateDescriptor(PVNet_ADAPTER Adapter, ULONG TotalBytes, USHORT StartOfTxBuffer, BOOLEAN IsData);

VOID
GetTxDescFlags(PVNet_ADAPTER Adapter, USHORT Descriptor, PUCHAR flags);


VOID 
GetTxDescStatus(PVNet_ADAPTER Adapter, USHORT Descriptor, PUCHAR status);

VOID 
SetTxDescFlags(PVNet_ADAPTER Adapter, USHORT Descriptor, UCHAR flags);

VOID 
SetTxDescRetryCount(PVNet_ADAPTER Adapter, USHORT Descriptor, UCHAR cnt);

VOID
SetTxDescNext(PVNet_ADAPTER Adapter, USHORT Descriptor, ULONG next);

VOID 
SetTxDescMsduPos(PVNet_ADAPTER Adapter, USHORT Descriptor, USHORT msdupos);

VOID 
GetTxDescMsduSize(PVNet_ADAPTER Adapter, USHORT Descriptor, PUSHORT msdusize);

VOID 
SetTxDescMsduSize(PVNet_ADAPTER Adapter, USHORT Descriptor, USHORT msdusize);

VOID 
SetTxDescRate(PVNet_ADAPTER Adapter, USHORT Descriptor, UCHAR RateIndex);

VOID 
GetTxDescHostMsduSize(PVNet_ADAPTER Adapter, USHORT Descriptor, PUSHORT msdusize);

VOID 
SetTxDescHostMsduSize(PVNet_ADAPTER Adapter, USHORT Descriptor, USHORT msdusize);

VOID 
GetTxDescPacketType(PVNet_ADAPTER Adapter, USHORT Descriptor, PUCHAR PacketType);


VOID
SetTxDescPacketType(PVNet_ADAPTER Adapter, USHORT Descriptor, UCHAR PacketType);

////////
// inlines.c
////////

VOID
CardMemToHostMem16(PVNet_ADAPTER Adapter, PUCHAR pDestBuf, ULONG SramAd, ULONG Bytes);

VOID
HostMemToCardMem16(PVNet_ADAPTER Adapter, ULONG SramAd, PUCHAR pSrcBuf, ULONG Bytes);

VOID 
SetFuncCtrl(PVNet_ADAPTER Adapter, PUCHAR FuncCtrl);

VOID 
SetLockoutMac(PVNet_ADAPTER Adapter, PUCHAR pData);

VOID 
SetIntStatus(PVNet_ADAPTER Adapter, PUCHAR IntStatus);

VOID 
SetIntMask(PVNet_ADAPTER Adapter, PUCHAR IntMask);

VOID 
GetLockoutHost(PVNet_ADAPTER Adapter, PUCHAR pData);


VOID 
GetLockoutMac(PVNet_ADAPTER Adapter, PUCHAR pData);

VOID 
GetIntStatus(PVNet_ADAPTER Adapter, PUCHAR IntStatus);

VOID
GetFuncCtrl(PVNet_ADAPTER Adapter, PUCHAR FuncCtrl);

#ifdef WINDOWS_OS
VOID 
GetMacStatus(PVNet_ADAPTER Adapter, PMAC_STATUS MacStatus);
#endif

int 
MemCompare(PUCHAR pMem1, PUCHAR pMem2, ULONG bytes);

VOID 
IndicateConnectionStatus(PVNet_ADAPTER Adapter, BOOLEAN Connected);

VOID 
CardReadUshort(PVNet_ADAPTER Adapter, UCHAR Offset, PUSHORT pData);

VOID
CardWriteUshort(PVNet_ADAPTER Adapter, UCHAR Offset, USHORT Data);

VOID 
CardWriteUchar(PVNet_ADAPTER Adapter, UCHAR Offset, UCHAR Data);

VOID 
CardReadUchar(PVNet_ADAPTER Adapter, UCHAR Offset, PUCHAR pData);

VOID 
CardMemToHostMem8(PVNet_ADAPTER Adapter, PUCHAR pDestBuf, ULONG SramAd, ULONG Bytes);

VOID 
HostMemToCardMem8(PVNet_ADAPTER Adapter, ULONG SramAd, PUCHAR pSrcBuf, ULONG Bytes);

///////
// interrupt.c
///////

VOID 
DisableInterrupts(PVNet_ADAPTER Adapter);

VOID 
EnableInterrupts(PVNet_ADAPTER Adapter);

VOID 
AcknowledgeInterrupt(PVNet_ADAPTER Adapter);

UCHAR 
GetInterruptStatus(PVNet_ADAPTER Adapter);

UCHAR
ResetInterruptStatusBit(PVNet_ADAPTER Adapter, UCHAR mask);

VOID
SetInterruptMask(PVNet_ADAPTER Adapter, UCHAR Mask);

//INTERRUPT_TYPE 
//GetInterruptType(UCHAR IntStatus);

VOID 
ProcessInterrupt(PVNet_ADAPTER Adapter);
////////
// rx.c
////////

#ifdef SNIFFER
VOID 
RxSnifferComplete(PVNet_ADAPTER Adapter);
#endif

VOID 
RxCompleteDPC(PVNet_ADAPTER Adapter);

#ifdef RX_CRC
VOID 
init_CRCtable(PVNet_ADAPTER Adapter);

ULONG 
Calculate_CRC32(PUCHAR p, ULONG FrameLength, PULONG crctab);

#endif //RX_CRC

////////////////
// interrupt.c
///////////////
VOID
ProcessInterrupt(PVNet_ADAPTER Adapter);

VOID
VNetMoveMemory(PVOID pDest, PVOID pSrc, ULONG bytes);

VOID
VNetZeroMemory(PUCHAR buf, ULONG bytes);


VOID CardReadUshort(PVNet_ADAPTER Adapter, UCHAR Offset, PUSHORT pData);

#ifdef LINUX_OS

VOID DbgPrint(const char *fmt,...);

#endif


#ifdef CE_OID

BOOL LoadProtocolDriver(VOID);
BOOL IsProtocolRunning(VOID);
BOOL GetValueFromSubKey(PWCHAR SubKey, PWCHAR ValueName, PBYTE OutBuf, PDWORD pOutBufLen) ;

#endif

#endif
